@@ -1,7 +1,7 @@
 /*
  * examples/libsyd_test.c: Build & link against libsyd + libcheck + yajl.
  * Run under syd(2) with `lib` profile by the Rust test harness.
- * Copyright (c) 2025 Ali Polatel <alip@chesswob.org>
+ * Copyright (c) 2025, 2026 Ali Polatel <alip@chesswob.org>
  * SPDX-License-Identifier: LGPL-3.0
  */
 
@@ -1038,12 +1038,15 @@ END_TEST
 
 START_TEST(test_lock)
 {
-	ck_assert_int_eq(syd_lock(LOCK_OFF), 0);
-	ck_assert_int_eq(syd_lock(LOCK_EXEC), 0);
+	ck_assert_int_eq(syd_lock(LOCK_OFF), -EPERM);
+	ck_assert_int_eq(syd_lock(LOCK_EXEC), 0); // no-op
+	ck_assert_int_eq(syd_lock(LOCK_DROP), 0);
 	ck_assert_int_eq(syd_lock(LOCK_ON), 0);
 
 	ck_assert_int_eq(syd_lock(LOCK_OFF), -ENOENT);
 	ck_assert_int_eq(syd_lock(LOCK_EXEC), -ENOENT);
+	ck_assert_int_eq(syd_lock(LOCK_DROP), -ENOENT);
+	ck_assert_int_eq(syd_lock(LOCK_READ), -ENOENT);
 	ck_assert_int_eq(syd_lock(LOCK_ON), -ENOENT);
 }
 END_TEST
