@@ -2,7 +2,7 @@
 // libsyd: Rust-based C library for syd interaction via /dev/syd
 // lib/src/lib.rs: syd API C Library
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: LGPL-3.0
 
@@ -87,8 +87,14 @@ pub const LOCK_OFF: lock_state_t = 0;
 /// The sandbox lock is set to on for all processes except the initial
 /// process (syd exec child).
 pub const LOCK_EXEC: lock_state_t = 1;
+/// The sandbox lock is in drop-only mode, allowing only privilege-dropping
+/// sandbox commands.
+pub const LOCK_DROP: lock_state_t = 2;
+/// The sandbox lock is in read-only mode, allowing only read-only access
+/// to sandbox state.
+pub const LOCK_READ: lock_state_t = 3;
 /// The sandbox lock is on, disallowing all sandbox commands.
-pub const LOCK_ON: lock_state_t = 2;
+pub const LOCK_ON: lock_state_t = 4;
 
 // An enumeration of the possible states for the sandbox lock.
 #[repr(u8)]
@@ -99,6 +105,12 @@ enum LockState {
     // The sandbox lock is set to on for all processes except the initial
     // process (syd exec child).
     Exec = LOCK_EXEC,
+    // The sandbox lock is in drop-only mode, allowing only privilege-dropping
+    // sandbox commands.
+    Drop = LOCK_DROP,
+    // The sandbox lock is in read-only mode, allowing only read-only access
+    // to sandbox state.
+    Read = LOCK_READ,
     // The sandbox lock is on, disallowing all sandbox commands.
     On = LOCK_ON,
 }
@@ -110,6 +122,8 @@ impl TryFrom<lock_state_t> for LockState {
         match value {
             LOCK_OFF => Ok(LockState::Off),
             LOCK_EXEC => Ok(LockState::Exec),
+            LOCK_DROP => Ok(LockState::Drop),
+            LOCK_READ => Ok(LockState::Read),
             LOCK_ON => Ok(LockState::On),
             _ => Err(()),
         }
@@ -121,6 +135,8 @@ impl fmt::Display for LockState {
         let state_str = match self {
             LockState::Off => "off",
             LockState::Exec => "exec",
+            LockState::Drop => "drop",
+            LockState::Read => "read",
             LockState::On => "on",
         };
         write!(f, "{state_str}")
@@ -858,6 +874,30 @@ pub extern "C" fn syd_disable_chroot() -> c_int {
     stat("/dev/syd/sandbox/chroot:off")
 }
 
+/// Checks if notify sandboxing is enabled.
+///
+/// Returns true if notify sandboxing is enabled, false otherwise.
+#[no_mangle]
+pub extern "C" fn syd_enabled_notify() -> bool {
+    stat("/dev/syd/sandbox/notify?") == 0
+}
+
+/// Enable notify sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_enable_notify() -> c_int {
+    stat("/dev/syd/sandbox/notify:on")
+}
+
+/// Disable notify sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_disable_notify() -> c_int {
+    stat("/dev/syd/sandbox/notify:off")
+}
+
 /// Checks if utime sandboxing is enabled.
 ///
 /// Returns true if utime sandboxing is enabled, false otherwise.
@@ -1353,6 +1393,17 @@ pub extern "C" fn syd_default_chroot(action: action_t) -> c_int {
     stat(&format!("/dev/syd/default/chroot:{action}"))
 }
 
+/// Set the default action for Notify Sandboxing.
+#[no_mangle]
+pub extern "C" fn syd_default_notify(action: action_t) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+    stat(&format!("/dev/syd/default/notify:{action}"))
+}
+
 /// Set the default action for Utime Sandboxing.
 #[no_mangle]
 pub extern "C" fn syd_default_utime(action: action_t) -> c_int {
@@ -2492,6 +2543,52 @@ pub extern "C" fn syd_chroot_rem(action: action_t, glob: *const c_char) -> c_int
     esyd(&format!("{action}/chroot"), glob, b'^')
 }
 
+/// Adds to the given actionlist of notify sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_notify_add(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with add operator.
+    esyd(&format!("{action}/notify"), glob, b'+')
+}
+
+/// Removes the first instance from the end of the given actionlist of
+/// notify sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_notify_del(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with del operator.
+    esyd(&format!("{action}/notify"), glob, b'-')
+}
+
+/// Removes all matching patterns from the given actionlist of notify sandboxing.
+///
+/// Returns 0 on success, negated errno on failure.
+#[no_mangle]
+pub extern "C" fn syd_notify_rem(action: action_t, glob: *const c_char) -> c_int {
+    // Convert action_t enum to corresponding action string.
+    let action = match Action::try_from(action) {
+        Ok(action) => action,
+        Err(_) => return -EINVAL,
+    };
+
+    // Call magic function with rem operator.
+    esyd(&format!("{action}/notify"), glob, b'^')
+}
+
 /// Adds to the given actionlist of utime sandboxing.
 ///
 /// Returns 0 on success, negated errno on failure.
