@@ -2,25 +2,19 @@
 // Syd: rock-solid application kernel
 // src/kernel/chmod.rs: chmod(2), fchmod(2), fchmodat(2), and fchmodat2(2) handlers
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
-use std::{
-    os::fd::{AsFd, AsRawFd},
-    sync::LazyLock,
-};
+use std::os::fd::AsRawFd;
 
-use libseccomp::{ScmpNotifResp, ScmpSyscall};
-use nix::{
-    errno::Errno,
-    fcntl::AtFlags,
-    sys::stat::{fchmod, Mode},
-    NixPath,
-};
+use libseccomp::ScmpNotifResp;
+use nix::{errno::Errno, fcntl::AtFlags, sys::stat::Mode};
 
 use crate::{
-    config::PROC_FD,
+    config::PROC_FILE,
+    cookie::{safe_fchmod, safe_fchmodat, safe_fchmodat2},
+    fd::fd_mode,
     kernel::{syscall_path_handler, to_atflags, to_mode},
     lookup::FsFlags,
     path::XPathBuf,
@@ -28,41 +22,42 @@ use crate::{
     sandbox::SandboxGuard,
 };
 
-// Note fchmodat2 may not be available,
-// and libc::SYS_fchmodat2 may not be defined.
-// Therefore we query the number using libseccomp.
-static SYS_FCHMODAT2: LazyLock<libc::c_long> = LazyLock::new(|| {
-    ScmpSyscall::from_name("fchmodat2")
-        .map(i32::from)
-        .map(libc::c_long::from)
-        .unwrap_or(-1) // Invalid system call.
-});
-
 pub(crate) fn sys_fchmod(request: UNotifyEventRequest) -> ScmpNotifResp {
     let req = request.scmpreq;
 
     // SAFETY: Strip undefined/invalid mode bits.
     let mode = to_mode(req.data.args[1]);
 
-    // WANT_READ: fchmod(2) does not work with O_PATH fds.
     let argv = &[SysArg {
         dirfd: Some(0),
-        fsflags: FsFlags::MUST_PATH | FsFlags::WANT_READ,
+        fsflags: FsFlags::MUST_PATH,
         ..Default::default()
     }];
     syscall_path_handler(request, "fchmod", argv, |path_args, request, sandbox| {
         // SAFETY:
         // 1. SysArg has one element.
-        // 2. SysArg.path is None asserting dir is Some.
+        // 2. We use MUST_PATH, dir refers to the file.
         #[expect(clippy::disallowed_methods)]
-        let fd = path_args.0.as_ref().unwrap().dir.as_ref().unwrap();
+        let path = path_args.0.as_ref().unwrap();
+        let fd = path.dir();
 
-        // SAFETY: We apply force_umask to chmod modes to ensure consistency.
+        // SAFETY:
+        // 1. We apply force_umask to chmod(2) modes to ensure consistency.
+        // 2. Umask is only forced for regular files.
+        // 3. Sticky bit is immutable for directories unless trace/allow_unsafe_sticky:1 is set.
         let mut mode = mode;
-        let umask = sandbox.umask.unwrap_or(Mode::empty());
-        mode &= !umask;
+        if path.is_file() {
+            let umask = sandbox.umask.unwrap_or(Mode::empty());
+            mode &= !umask;
+        } else if !sandbox.flags.allow_unsafe_sticky()
+            && path.is_dir()
+            && fd_mode(fd)?.contains(Mode::S_ISVTX)
+        {
+            mode.insert(Mode::S_ISVTX);
+        }
+        drop(sandbox); // release the read-lock.
 
-        fchmod(fd, mode).map(|_| request.return_syscall(0))
+        safe_fchmod(fd, mode).map(|_| request.return_syscall(0))
     })
 }
 
@@ -78,7 +73,7 @@ pub(crate) fn sys_chmod(request: UNotifyEventRequest) -> ScmpNotifResp {
     }];
 
     syscall_path_handler(request, "chmod", argv, |path_args, request, sandbox| {
-        syscall_chmod_handler(request, &sandbox, path_args, mode)
+        syscall_chmod_handler(request, sandbox, path_args, mode)
     })
 }
 
@@ -96,7 +91,7 @@ pub(crate) fn sys_fchmodat(request: UNotifyEventRequest) -> ScmpNotifResp {
     }];
 
     syscall_path_handler(request, "fchmodat", argv, |path_args, request, sandbox| {
-        syscall_chmod_handler(request, &sandbox, path_args, mode)
+        syscall_chmod_handler(request, sandbox, path_args, mode)
     })
 }
 
@@ -125,59 +120,47 @@ pub(crate) fn sys_fchmodat2(request: UNotifyEventRequest) -> ScmpNotifResp {
     }];
 
     syscall_path_handler(request, "fchmodat2", argv, |path_args, request, sandbox| {
-        syscall_chmod_handler(request, &sandbox, path_args, mode)
+        syscall_chmod_handler(request, sandbox, path_args, mode)
     })
 }
 
 /// A helper function to handle chmod, fchmodat, and fchmodat2 syscalls.
 fn syscall_chmod_handler(
     request: &UNotifyEventRequest,
-    sandbox: &SandboxGuard,
+    sandbox: SandboxGuard,
     args: PathArgs,
     mut mode: Mode,
 ) -> Result<ScmpNotifResp, Errno> {
-    // SAFETY: SysArg has one element.
+    // SAFETY:
+    // 1. SysArg has one element.
+    // 2. We use MUST_PATH, dir refers to the file.
     #[expect(clippy::disallowed_methods)]
     let path = args.0.as_ref().unwrap();
+    let fd = path.dir();
 
-    // We use MUST_PATH, dir refers to the file.
-    assert!(
-        path.base().is_empty(),
-        "BUG: MUST_PATH returned a directory for chmod, report a bug!"
-    );
-    let fd = path.dir.as_ref().map(|fd| fd.as_fd()).ok_or(Errno::EBADF)?;
+    // SAFETY:
+    // 1. We apply force_umask to chmod modes to ensure consistency.
+    // 2. Umask is only forced for regular files.
+    // 3. Sticky bit is immutable for directories unless trace/allow_unsafe_sticky:1 is set.
+    if path.is_file() {
+        let umask = sandbox.umask.unwrap_or(Mode::empty());
+        mode &= !umask;
+    } else if !sandbox.flags.allow_unsafe_sticky()
+        && path.is_dir()
+        && fd_mode(fd)?.contains(Mode::S_ISVTX)
+    {
+        mode.insert(Mode::S_ISVTX);
+    }
+    drop(sandbox); // release the read-lock.
 
-    // SAFETY: We apply force_umask to chmod modes to ensure consistency.
-    let umask = sandbox.umask.unwrap_or(Mode::empty());
-    mode &= !umask;
-
-    match if *SYS_FCHMODAT2 > 0 {
-        // SAFETY: No libc wrapper for fchmodat2 yet.
-        Errno::result(unsafe {
-            libc::syscall(
-                *SYS_FCHMODAT2,
-                fd.as_raw_fd(),
-                c"".as_ptr(),
-                mode.bits(),
-                libc::AT_EMPTY_PATH,
-            )
-        })
-    } else {
-        Err(Errno::ENOSYS)
-    } {
+    match safe_fchmodat2(fd, mode) {
         Ok(_) => Ok(()),
         Err(Errno::ENOSYS) => {
             // Fallback to `/proc` indirection,
             //
             // path to fd is open already!
             let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
-            pfd.with_nix_path(|cstr| {
-                // SAFETY: We deliberately bypass the libc wrapper here.
-                Errno::result(unsafe {
-                    libc::syscall(libc::SYS_fchmodat, PROC_FD(), cstr.as_ptr(), mode.bits())
-                })
-            })?
-            .map(drop)
+            safe_fchmodat(PROC_FILE(), &pfd, mode)
         }
         Err(errno) => Err(errno),
     }
