@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/pool.rs: Self growing / shrinking `ThreadPool` implementation
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon rusty_pool which is:
 //     Copyright (c) Robin Friedli <robinfriedli@icloud.com>
 //     SPDX-License-Identifier: Apache-2.0
@@ -14,7 +14,6 @@
 // Commit:d56805869ba3cbe47021d5660bbaf19ac5ec4bfb
 
 use std::{
-    env,
     fs::OpenOptions,
     io::Write,
     option::Option,
@@ -42,16 +41,16 @@ use nix::{
 use crate::{
     alert,
     config::*,
-    confine::ExportMode,
+    confine::{secure_getenv, ExportMode},
     debug,
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
-    fs::{block_signal, closeexcept, seccomp_export_pfc},
+    fs::{block_signal, seccomp_export_pfc},
     hook::HandlerMap,
     info,
     retry::{retry_on_eintr, retry_on_intr},
     rwrite, rwriteln,
-    sandbox::{Capability, Flags, Sandbox},
+    sandbox::{Capability, Options, Sandbox},
     workers::{
         aes::{AesLock, AesWorker},
         emu::Worker,
@@ -70,7 +69,7 @@ extern "C" fn handle_sigalrm(_: libc::c_int) {}
 pub(crate) struct ThreadPool {
     core_size: usize,
     keep_alive: u16,
-    flags: Flags,
+    options: Options,
     seccomp_fd: RawFd,
     pub(crate) cache: Arc<WorkerCache>,
     sandbox: Arc<RwLock<Sandbox>>,
@@ -92,7 +91,7 @@ impl ThreadPool {
     #[expect(clippy::too_many_arguments)]
     pub(crate) fn new(
         seccomp_fd: RawFd,
-        flags: Flags,
+        options: Options,
         core_size: usize,
         keep_alive: u16,
         sandbox: Arc<RwLock<Sandbox>>,
@@ -105,7 +104,7 @@ impl ThreadPool {
             handlers,
             core_size,
             keep_alive,
-            flags,
+            options,
             should_exit,
             seccomp_fd,
             cache: Arc::new(WorkerCache::new(crypt_map)),
@@ -123,7 +122,8 @@ impl ThreadPool {
             Some(ExportMode::BerkeleyPacketFilter) => {
                 // Worker rules
                 let is_crypt = self.cache.crypt_map.is_some();
-                let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
+                let ctx =
+                    Worker::prepare_confine(self.seccomp_fd, self.options, is_crypt, &[], &[])?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -136,7 +136,7 @@ impl ThreadPool {
                 let ctx = Interrupter::prepare_confine(
                     self.seccomp_fd,
                     getpid(),
-                    self.flags,
+                    self.options,
                     &[],
                     &[],
                     true,
@@ -151,7 +151,7 @@ impl ThreadPool {
                 // IPC thread rules
                 // We pass dummy RawFd=2525 for epoll FD.
                 // We pass dry_run=true to avoid Landlock confinement.
-                let ctx = IpcWorker::prepare_confine(2525, self.flags, &[], &[], true)?;
+                let ctx = IpcWorker::prepare_confine(2525, self.options, &[], &[], true)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -160,7 +160,7 @@ impl ThreadPool {
                 ctx.export_bpf(file)?;
 
                 // Aes worker rules
-                let ctx = AesWorker::prepare_confine(self.flags, &[], &[])?;
+                let ctx = AesWorker::prepare_confine(self.options, &[], &[], true)?;
                 let file = OpenOptions::new()
                     .write(true)
                     .create_new(true)
@@ -178,7 +178,8 @@ impl ThreadPool {
                     self.seccomp_fd
                 )?;
                 let is_crypt = self.cache.crypt_map.is_some();
-                let ctx = Worker::prepare_confine(self.seccomp_fd, self.flags, is_crypt, &[], &[])?;
+                let ctx =
+                    Worker::prepare_confine(self.seccomp_fd, self.options, is_crypt, &[], &[])?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
                 // We pass dry_run=true to avoid Landlock confinement.
@@ -190,7 +191,7 @@ impl ThreadPool {
                 let ctx = Interrupter::prepare_confine(
                     self.seccomp_fd,
                     getpid(),
-                    self.flags,
+                    self.options,
                     &[],
                     &[],
                     true,
@@ -200,11 +201,11 @@ impl ThreadPool {
                 // We pass dummy RawFd=2525 for epoll FD.
                 // We pass dry_run=true to avoid Landlock confinement.
                 rwriteln!(stdout, "# Syd ipc rules")?;
-                let ctx = IpcWorker::prepare_confine(2525, self.flags, &[], &[], true)?;
+                let ctx = IpcWorker::prepare_confine(2525, self.options, &[], &[], true)?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
 
                 rwriteln!(stdout, "# Syd encryptor rules")?;
-                let ctx = AesWorker::prepare_confine(self.flags, &[], &[])?;
+                let ctx = AesWorker::prepare_confine(self.options, &[], &[], true)?;
                 rwrite!(stdout, "{}", seccomp_export_pfc(&ctx)?)?;
             }
             _ => {}
@@ -242,7 +243,13 @@ impl ThreadPool {
 
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
-                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES) {
+                // Skip CLONE_FILES for KCOV because ptrace handler must close FDs.
+                let unshare_flags = if !cfg!(feature = "kcov") {
+                    CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES
+                } else {
+                    CloneFlags::CLONE_FS
+                };
+                if let Err(errno) = unshare(unshare_flags) {
                     alert!("ctx": "boot", "op": "unshare_monitor_thread",
                         "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES): {errno}"),
                         "err": errno as i32);
@@ -275,7 +282,8 @@ impl ThreadPool {
                     }
                 }
                 set.sort_unstable();
-                closeexcept(&set)?;
+                #[cfg(not(feature = "kcov"))]
+                crate::fd::closeexcept(&set)?;
                 drop(set);
 
                 // Spawn the interrupt thread which will confine itself.
@@ -315,19 +323,22 @@ impl ThreadPool {
                 //    as it is inherited.
                 // 2. If sandbox is not locked yet, build the seccomp context anyway,
                 //    precompute it and pass it to emulator threads for fast confinement.
-                let dry_run = env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
-                let safe_setid = self.flags.intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+                // 3. If sandbox is locked, and trace/force_umask is set, use it to confine
+                //    fchmodat(2) and fchmodat2(2) mode argument.
+                let dry_run = secure_getenv(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
+                let safe_setid = self.options.intersects(Options::OPT_ALLOW_SAFE_SETUID | Options::OPT_ALLOW_SAFE_SETGID);
                 let is_crypt = self.cache.crypt_map.is_some();
+                let is_locked = sandbox.is_locked();
 
                 let mut ctx = if !dry_run {
                     let ctx = Worker::prepare_confine(
                         self.seccomp_fd,
-                        self.flags,
+                        self.options,
                         is_crypt,
                         &sandbox.transit_uids,
                         &sandbox.transit_gids)?;
 
-                    if Sandbox::locked_once() {
+                    if is_locked {
                         // Sandbox locked, confine right away.
                         //
                         // SAFETY: We use exit_group(2) here to bail,
@@ -388,7 +399,7 @@ impl ThreadPool {
                 loop {
                     // Confine and drop filter if sandbox is locked.
                     if let Some(ref filter) = ctx {
-                        if Sandbox::locked_once() {
+                        if Sandbox::is_locked_once() {
                             // SAFETY: We use exit_group(2) here to bail,
                             // because this confinement is a critical safety feature.
                             if let Err(error) = filter.load() {
@@ -485,7 +496,7 @@ impl ThreadPool {
         let handle = retry_on_intr(|| {
             Interrupter::new(
                 self.seccomp_fd,
-                self.flags,
+                self.options,
                 transit_uids,
                 transit_gids,
                 Arc::clone(&self.should_exit),
@@ -528,7 +539,7 @@ impl ThreadPool {
         let handle = retry_on_intr(|| {
             Timeouter::new(
                 tmout,
-                self.flags,
+                self.options,
                 transit_uids,
                 transit_gids,
                 Arc::clone(&self.should_exit),
@@ -567,7 +578,7 @@ impl ThreadPool {
         let worker = AesWorker::new(
             fdalg,
             files,
-            self.flags,
+            self.options,
             memfd,
             should_exit,
             transit_uids,
@@ -605,11 +616,6 @@ impl ThreadPool {
             None
         } else if busy_worker_count < curr_worker_count {
             // We have idle threads, no need to spawn a new worker.
-            debug!("ctx": "spawn",
-                "msg": "idle emulator exists, no need to spawn",
-                "busy_worker_count": busy_worker_count,
-                "curr_worker_count": curr_worker_count,
-                "core_size": self.core_size);
             return Ok(None);
         } else if curr_worker_count < *EMU_MAX_SIZE {
             // Create a new helper worker if the current worker count is
@@ -656,6 +662,7 @@ impl ThreadPool {
     fn signal_int(&self) {
         // Unblock stuck emulator threads with manual signal.
         let mut nsig = 0usize;
+
         {
             let (ref lock, ref cvar) = *self.cache.sysint_map.sys_block;
             let mut map = lock.lock().unwrap_or_else(|err| err.into_inner());
@@ -671,10 +678,12 @@ impl ThreadPool {
             cvar.notify_one();
         }
 
-        alert!("ctx": "spawn",
-            "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
-            "nsig": nsig,
-            "core_size": self.core_size,
-            "keep_alive": self.keep_alive);
+        if nsig > 0 {
+            alert!("ctx": "spawn",
+                "msg": format!("signaled interrupt to unstuck {nsig} emulators"),
+                "nsig": nsig,
+                "core_size": self.core_size,
+                "keep_alive": self.keep_alive);
+        }
     }
 }
