@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-cat.rs: Print out the rules of the given sandbox profile and exit
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -23,6 +23,7 @@ use syd::{
     not(coverage),
     not(feature = "prof"),
     not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -64,7 +65,7 @@ syd::main! {
             Short('J') => optJ = true,
             Short('m') => {
                 let cmd = parser.value().map(XPathBuf::from)?;
-                if syd.locked() {
+                if syd.is_locked() {
                     eprintln!("syd-cat: Failed to execute magic command `{cmd}': sandbox locked!");
                     return Err(Errno::EBUSY.into());
                 } else {
@@ -119,9 +120,9 @@ syd::main! {
     }
 
     for cmd in optM {
-        if syd.locked() {
+        if syd.is_locked() {
             eprintln!("syd-cat: Failed to execute magic command `{cmd}': sandbox locked!");
-            return Err(Errno::EBUSY.into());
+            return Err(Errno::EPERM.into());
         } else {
             syd.config(&cmd)?;
         }
@@ -158,6 +159,7 @@ fn help() {
 }
 
 fn list() {
+    println!("chrome");
     println!("container");
     println!("core");
     println!("cwd");
@@ -165,17 +167,23 @@ fn list() {
     println!("enforce");
     println!("firefox");
     println!("fs");
+    println!("gui");
     println!("hide");
     println!("immutable");
     println!("landlock");
+    println!("lang");
+    println!("ldd");
     println!("lib");
     println!("linux");
     println!("ltp");
     println!("nix");
+    println!("nixstore");
     println!("noipv4");
     println!("noipv6");
+    println!("nomagic");
     println!("nomem");
     println!("nopie");
+    println!("noxdev");
     println!("oci");
     println!("paludis");
     println!("quiet");
@@ -183,6 +191,7 @@ fn list() {
     println!("readonly");
     println!("tty");
     println!("user");
+    println!("xdg");
 }
 
 fn dump(name: &str) -> SydResult<()> {
@@ -238,6 +247,15 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "kcov" => {
+            println!("# Syd profile: KCOV");
+            println!("# Number of rules: {}", syd::config::PROFILE_KCOV.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_KCOV {
+                println!("{command}");
+            }
+        }
         "kvm" => {
             println!("# Syd profile: KVM");
             println!("# Number of rules: {}", syd::config::PROFILE_KVM.len());
@@ -391,6 +409,24 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "nomagic" => {
+            println!("# Syd profile: Enforce No Magic Links");
+            println!("# Number of rules: {}", syd::config::PROFILE_NO_MAGIC.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_NO_MAGIC {
+                println!("{command}");
+            }
+        }
+        "noxdev" => {
+            println!("# Syd profile: Enforce No Cross Mounts");
+            println!("# Number of rules: {}", syd::config::PROFILE_NO_XDEV.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_NO_XDEV {
+                println!("{command}");
+            }
+        }
         "quiet" | "silent" => {
             println!("# Syd profile: Quiet");
             println!("# Number of rules: {}", syd::config::PROFILE_QUIET.len());
@@ -418,6 +454,15 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "chrome" => {
+            println!("# Syd profile: Chrome-family browsers");
+            println!("# Number of rules: {}", syd::config::PROFILE_CHROME.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_CHROME {
+                println!("{command}");
+            }
+        }
         "ff" | "firefox" => {
             println!("# Syd profile: Firefox");
             println!("# Number of rules: {}", syd::config::PROFILE_FIREFOX.len());
@@ -436,6 +481,42 @@ fn dump(name: &str) -> SydResult<()> {
                 println!("{command}");
             }
         }
+        "lang" => {
+            println!("# Syd profile: Allow language and timezone environment variables");
+            println!("# Number of rules: {}", syd::config::PROFILE_LANG.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_LANG {
+                println!("{command}");
+            }
+        }
+        "gui" => {
+            println!("# Syd profile: Graphical User Interface");
+            println!("# Number of rules: {}", syd::config::PROFILE_GUI.len());
+            println!("# Copyright (c) 2025 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_GUI {
+                println!("{command}");
+            }
+        }
+        "xdg" => {
+            println!("# Syd profile: Allow XDG environment variables");
+            println!("# Number of rules: {}", syd::config::PROFILE_XDG.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_XDG {
+                println!("{command}");
+            }
+        }
+        "ldd" => {
+            println!("# Syd profile: Allow ldd(1)");
+            println!("# Number of rules: {}", syd::config::PROFILE_LDD.len());
+            println!("# Copyright (c) 2026 Ali Polatel <alip@chesswob.org>");
+            println!("# SPDX-License-Identifier: GPL-3.0");
+            for command in syd::config::PROFILE_LDD {
+                println!("{command}");
+            }
+        }
         "off" => {
             println!("# Syd profile: Off");
             println!("# Number of rules: {}", syd::config::PROFILE_OFF.len());
