@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // src/utils/syd-oci.rs: syd's OCI container runtime
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Based in part upon youki which is:
 //     Copyright (c) 2021 youki team
 //     SPDX-License-Identifier: Apache-2.0
@@ -80,8 +80,9 @@ use oci_spec::runtime::{
 use procfs::process::Namespace;
 use serde_json::to_writer_pretty;
 use syd::{
-    compat::{set_name, waitid, WaitStatus},
+    compat::{set_name, set_no_new_privs, waitid, WaitStatus},
     config::ENV_SKIP_SCMP,
+    confine::secure_getenv,
     err::{SydError, SydResult},
     hook::Supervisor,
     ignore_signals,
@@ -99,6 +100,7 @@ use tracing_subscriber::layer::SubscriberExt;
     not(coverage),
     not(feature = "prof"),
     not(target_os = "android"),
+    not(target_arch = "riscv64"),
     target_page_size_4k,
     target_pointer_width = "64"
 ))]
@@ -202,13 +204,13 @@ impl Executor for SydExecutor {
         // Ignore all signals except SIG{CHLD,KILL,STOP}.
         // This is used to ensure we can deny {rt_,}sigreturn(2) to mitigate SROP.
         let mut opts = IgnoreSignalOpts::SkipIgnoreAlarm;
-        if sandbox.flags.allow_unsafe_prlimit() {
+        if sandbox.options.allow_unsafe_prlimit() {
             opts.insert(IgnoreSignalOpts::SkipIgnoreCoreDump);
         }
         ignore_signals(opts).or(Err(ExecutorError::CantHandle("Failed to ignore signals!")))?;
 
         // Run command under the Syd sandbox and exit with the return code.
-        Supervisor::run(sandbox, None, comm, argv, None, None)
+        Supervisor::run(sandbox, None, comm, argv, None)
             .map(i32::from)
             .map(exit)
             .map_err(|err| ExecutorError::Execution(err.into()))?
@@ -1483,6 +1485,18 @@ struct Opts {
 }
 
 syd::main! {
+    // Set NO_NEW_PRIVS as early as possible.
+    set_no_new_privs()?;
+
+    // Apply a landlock(7) scope sandbox to restrict:
+    // 1. ptrace(2) attach outside landlock(7).
+    // 2. Signal send outside landlock(7).
+    // 3. We leave path and network restrictions for Landlock
+    //    to be configured by the user using Lock sandboxing.
+    // 4. We do this before memfd-reexec to add an additional
+    //    guard against proc(5) havoc.
+    syd::confine::confine_landlock_scope()?;
+
     // Guard against CVE-2019-5736:
     // Copy /proc/self/exe in an anonymous fd (created via memfd_create), seal it and re-execute it.
     // See:
@@ -1498,7 +1512,7 @@ syd::main! {
             Some(val) => env::set_var("SYD_RUST_BACKTRACE", val),
             None => env::remove_var("SYD_RUST_BACKTRACE"),
         };
-        if env::var_os(ENV_SKIP_SCMP).is_none() {
+        if secure_getenv(ENV_SKIP_SCMP).is_none() {
             env::set_var("RUST_BACKTRACE", "0");
         }
     }
