@@ -1,8 +1,8 @@
 //
 // Syd: rock-solid application kernel
-// src/syd-rnd.rs: print AT_RANDOM bytes in various formats
+// src/utils/syd-rnd.rs: print AT_RANDOM bytes in various formats
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
@@ -11,6 +11,23 @@ use std::{io::Write, process::ExitCode};
 use nix::errno::Errno;
 use syd::hash::{get_at_random, get_at_random_hex, get_at_random_name, get_at_random_u64};
 
+// Set global allocator to GrapheneOS allocator.
+#[cfg(all(
+    not(coverage),
+    not(feature = "prof"),
+    not(target_os = "android"),
+    not(target_arch = "riscv64"),
+    target_page_size_4k,
+    target_pointer_width = "64"
+))]
+#[global_allocator]
+static GLOBAL: hardened_malloc::HardenedMalloc = hardened_malloc::HardenedMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     use lexopt::prelude::*;
 
