@@ -2,12 +2,11 @@
 // Syd: rock-solid application kernel
 // src/workers/out.rs: `syd_out' timeouter thread
 //
-// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 //
 // SPDX-License-Identifier: GPL-3.0
 
 use std::{
-    env,
     os::fd::{FromRawFd, OwnedFd, RawFd},
     sync::{
         atomic::{AtomicBool, Ordering},
@@ -17,7 +16,7 @@ use std::{
 };
 
 use dur::Duration;
-use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use libseccomp::{ScmpAction, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
     sched::{unshare, CloneFlags},
@@ -27,7 +26,10 @@ use nix::{
 use crate::{
     alert,
     config::*,
-    confine::{confine_scmp_madvise, confine_scmp_write, scmp_add_setid_rules, ExportMode},
+    confine::{
+        confine_scmp_fcntl, confine_scmp_madvise, confine_scmp_write, confine_scmp_wx_syd,
+        scmp_add_setid_rules, secure_getenv, ExportMode,
+    },
     err::{err2no, scmp2no, SydJoinHandle, SydResult},
     error,
     fd::closeexcept,
@@ -35,13 +37,13 @@ use crate::{
     landlock::Errata,
     landlock_policy::LandlockPolicy,
     retry::retry_on_eintr,
-    sandbox::Flags,
+    sandbox::Options,
 };
 
 #[derive(Clone)]
 pub(crate) struct Timeouter {
     tmout: Duration,
-    flags: Flags,
+    options: Options,
 
     transit_uids: Vec<(Uid, Uid)>,
     transit_gids: Vec<(Gid, Gid)>,
@@ -52,14 +54,14 @@ pub(crate) struct Timeouter {
 impl Timeouter {
     pub(crate) fn new(
         tmout: Duration,
-        flags: Flags,
+        options: Options,
         transit_uids: &[(Uid, Uid)],
         transit_gids: &[(Gid, Gid)],
         should_exit: Arc<AtomicBool>,
     ) -> Self {
         Self {
             tmout,
-            flags,
+            options,
             should_exit,
             transit_uids: transit_uids.to_vec(),
             transit_gids: transit_gids.to_vec(),
@@ -74,9 +76,9 @@ impl Timeouter {
             .spawn(move || {
                 // SAFETY: We use exit_group(2) here to bail,
                 // because this unsharing is a critical safety feature.
-                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES) {
+                if let Err(errno) = unshare(CloneFlags::CLONE_FS | CloneFlags::CLONE_FILES | CloneFlags::CLONE_SYSVSEM) {
                     alert!("ctx": "boot", "op": "unshare_timeout_thread",
-                        "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES): {errno}"),
+                        "msg": format!("failed to unshare(CLONE_FS|CLONE_FILES |CLONE_SYSVSEM): {errno}"),
                         "err": errno as i32);
                     std::process::exit(101);
                 }
@@ -113,14 +115,14 @@ impl Timeouter {
 
                 // Honour dry-run when exporting.
                 let dry_run =
-                    env::var_os(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
+                    secure_getenv(ENV_SKIP_SCMP).is_some() || ExportMode::from_env().is_some();
 
                 // Confine `syd_out' thread.
                 if !dry_run {
                     // SAFETY: We use exit_group(2) here to bail,
                     // because this confinement is a critical safety feature.
                     let ctx = match Self::prepare_confine(
-                        self.flags,
+                        self.options,
                         &self.transit_uids,
                         &self.transit_gids,
                         false,
@@ -147,8 +149,8 @@ impl Timeouter {
                     }
 
                     let safe_setid = self
-                        .flags
-                        .intersects(Flags::FL_ALLOW_SAFE_SETUID | Flags::FL_ALLOW_SAFE_SETGID);
+                        .options
+                        .intersects(Options::OPT_ALLOW_SAFE_SETUID | Options::OPT_ALLOW_SAFE_SETGID);
                     info!("ctx": "confine", "op": "confine_out_thread",
                         "msg": format!("timeout thread confined with{} SROP mitigation",
                             if safe_setid { "out" } else { "" }));
@@ -182,7 +184,7 @@ impl Timeouter {
     /// Confine Timeouter thread.
     #[expect(clippy::cognitive_complexity)]
     pub(crate) fn prepare_confine(
-        flags: Flags,
+        options: Options,
         transit_uids: &[(Uid, Uid)],
         transit_gids: &[(Gid, Gid)],
         dry_run: bool,
@@ -208,7 +210,7 @@ impl Timeouter {
 
         // Disable Speculative Store Bypass mitigations
         // with trace/allow_unsafe_exec_speculative:1
-        ctx.set_ctl_ssb(flags.allow_unsafe_exec_speculative())?;
+        ctx.set_ctl_ssb(options.allow_unsafe_exec_speculative())?;
 
         // DO NOT synchronize filter to all threads.
         // Other threads will self-confine.
@@ -250,38 +252,10 @@ impl Timeouter {
         }
 
         // Allow safe fcntl(2) utility calls.
-        for sysname in ["fcntl", "fcntl64"] {
-            let syscall = match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => syscall,
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_out_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                    continue;
-                }
-            };
-
-            for op in OUT_FCNTL_OPS {
-                ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == *op)])?;
-            }
-        }
+        confine_scmp_fcntl(&mut ctx, OUT_FCNTL_OPS)?;
 
         // Prevent executable memory.
-        const PROT_EXEC: u64 = libc::PROT_EXEC as u64;
-        for sysname in ["mmap", "mmap2", "mprotect"] {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule_conditional(
-                        ScmpAction::Allow,
-                        syscall,
-                        &[scmp_cmp!($arg2 & PROT_EXEC == 0)],
-                    )?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_out_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
+        confine_scmp_wx_syd(&mut ctx)?;
 
         // Allow writes to the log-fd.
         // No proc_pid_mem(5) access required here.
@@ -291,33 +265,15 @@ impl Timeouter {
         confine_scmp_madvise(&mut ctx)?;
 
         // Allow safe system calls.
-        for sysname in OUT_SYSCALLS.iter().chain(VDSO_SYSCALLS) {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_out_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow futex system calls.
-        for sysname in FUTEX_SYSCALLS {
-            match ScmpSyscall::from_name(sysname) {
-                Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Allow, syscall)?;
-                }
-                Err(_) => {
-                    info!("ctx": "confine", "op": "allow_out_syscall",
-                        "msg": format!("invalid or unsupported syscall {sysname}"));
-                }
-            }
-        }
-
-        // Allow getid system calls.
-        for sysname in GET_ID_SYSCALLS {
+        //
+        // KCOV_SYSCALLS is empty in case `kcov` feature is disabled.
+        for sysname in OUT_SYSCALLS
+            .iter()
+            .chain(FUTEX_SYSCALLS)
+            .chain(GETID_SYSCALLS)
+            .chain(KCOV_SYSCALLS)
+            .chain(VDSO_SYSCALLS)
+        {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
                     ctx.add_rule(ScmpAction::Allow, syscall)?;
@@ -330,8 +286,8 @@ impl Timeouter {
         }
 
         // Allow UID/GID changing system calls as necessary.
-        let safe_setuid = flags.allow_safe_setuid();
-        let safe_setgid = flags.allow_safe_setgid();
+        let safe_setuid = options.allow_safe_setuid();
+        let safe_setgid = options.allow_safe_setgid();
         if safe_setuid || safe_setgid {
             scmp_add_setid_rules(
                 "out",
