@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // tests/tests.rs: Integration tests
 //
-// Copyright (c) 2023, 2024, 2025 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2023, 2024, 2025, 2026 Ali Polatel <alip@chesswob.org>
 // Copyright (c) 2023 Johannes Nixdorf <mixi@exherbo.org>
 //
 // SPDX-License-Identifier: GPL-3.0
@@ -31,7 +31,6 @@ const SYD_SEC_EXE: &'static str = env!("CARGO_BIN_EXE_syd-sec");
 const SYD_SIZE_EXE: &'static str = env!("CARGO_BIN_EXE_syd-size");
 const SYD_TEST_DO_EXE: &'static str = env!("CARGO_BIN_EXE_syd-test-do");
 const SYD_TEST_EXE: &'static str = env!("CARGO_BIN_EXE_syd-test");
-const SYD_TOR_EXE: &'static str = env!("CARGO_BIN_EXE_syd-tor");
 const SYD_UTS: &'static str = env!("CARGO_BIN_EXE_syd-uts");
 const SYD_X: &'static str = env!("CARGO_BIN_EXE_syd-x");
 
@@ -61,7 +60,6 @@ fn syd_test() {
     env::set_var("CARGO_BIN_EXE_syd-sec", SYD_SEC_EXE);
     env::set_var("CARGO_BIN_EXE_syd-size", SYD_SIZE_EXE);
     env::set_var("CARGO_BIN_EXE_syd-test-do", SYD_TEST_DO_EXE);
-    env::set_var("CARGO_BIN_EXE_syd-tor", SYD_TOR_EXE);
     env::set_var("CARGO_BIN_EXE_syd-uts", SYD_UTS);
     env::set_var("CARGO_BIN_EXE_syd-x", SYD_X);
 
